## The GENERATE Tool is created by NA-ACCORD for local address geocoding and 
## submission of area-Level indicators

# Note: 
  # Before running this script, please read the "User Guideline.doc" file
  
# Set the Working Directory
# setwd("YOUR/PATH/HERE")

library(dplyr, quietly = TRUE)
library(lubridate)
library(tidyr)
library(tidyverse)

# Requirements of the input address csv file:
  # Should be named as "adrs_geocoded"
#The "adrs_geocoded.csv" file must contain: 
  # cohortPatientId (unique identifier of the input addresses)
  # lat (latitude)
  # lon (longitude)
  # month (month of the input address)
  # day (day of the input address)
  # year (year of the input address)
  # ZIP (zip code of the input addresses)
  
# Important: 
  # The csv file should have all these 7 columns
  # The name of each column should be exactly what's described above (i.e., cohortPatientId, Street Number, Street Name, etc.)
  # The names of the columns should be on the first row


# Import the input csv file
input_address <- read.csv("adrs_geocoded.csv") 


# Remove the addresses after year 2022
################ Note ##################
# We set it to 2022 because the latest data for now is by year 2022.
# This should be updated accordingly in the future.
input_address <- input_address |> filter(year<=2022)


# we want to have each year recorded for participants' address
# for participants with multiple address, startdate is the address recorded and end date is the day before next address record date
# for participants with single address recorded or the last address among the multiple address, enddate is the date for the last day of the protocol year
# then we want to separate each year in the duration between startdate and enddate to get census information for each year
input_address$record_date <- as.Date(paste(input_address$year, input_address$month, input_address$day, sep = "-"))

input_address <- input_address %>%
  arrange(cohortPatientId, record_date) %>% 
  group_by(cohortPatientId) %>%
  mutate(
    startdate = if_else(year(record_date) < 2012, as.Date("2012-01-01"), record_date),
    enddate = lead(record_date) - 1
  )  %>%
  ungroup()
# For patients with a single address or the last address among multiple addresses, set enddate to the last day of current protocol year
input_address$enddate[is.na(input_address$enddate)] <- as.Date("2022-12-31")

# write a function to expand rows by year
by_year <- function(start, end) {
  seq.Date(start, end, by = "year")
}

expand_years <- function(df) {
  df %>%
    mutate(year = map2(startdate, enddate, by_year)) %>%
    unnest(cols = c(year)) %>%
    mutate(
      start = if_else(year == startdate, startdate, floor_date(year, "year")),
      end = if_else(year == enddate, enddate, ceiling_date(year, "year") - days(1))
    ) %>%
    group_by(cohortPatientId) %>%
    mutate(lead_start = lead(start)) %>%
    mutate(end = if_else(!is.na(lead_start) & lead_start < end, lead_start - days(1), end)) %>%
    ungroup() %>%
    select(-startdate, -enddate, -year, -lead_start)
}


# expand rows by year
input_address <- expand_years(input_address)

# check for (end!=lead(start)-1) to see if still need to insert rows after expansion
insert_row <- input_address %>%
  group_by(cohortPatientId) %>%
  mutate(
    lead_start = lead(start),
    insert = !is.na(lead_start) & end != lead_start - days(1)
  ) %>%
  ungroup() %>%
  filter(insert) %>%
  mutate(end = lead_start - days(1)) %>%
  mutate(start = floor_date(end, "year")) %>%
  select(-lead_start, -insert)

# Combine the original expanded_df with the inserted rows
input_address <- bind_rows(input_address, insert_row) %>%
  arrange(cohortPatientId, start)

#change Year in input address to match the year of start date;
input_address <-input_address %>% mutate(year=year(start))


# Write the geocoded address into a csv file and save it to local folder
write.csv(input_address, file = "adrs_geocoded.csv", row.names = FALSE)


