## The GENERATE Tool is created by NA-ACCORD for local address geocoding and 
## submission of area-Level indicators

# This R script aims to perturb the last digit of the pulled estimates
# This should be run before the indicated point in "pull_census.R"
#

# Load the necessary libraries
library(dplyr, quietly = TRUE)
library(tidyverse, quietly = TRUE)


# Import the required csv file
output <- read.csv("temp_output.csv") 

# State median income
# Generate a list of random digits used to replace the last digits of the estimates 
state_med_inc_random <- sample(0:9, length(output$mStateIncome), replace = TRUE)
num_state_med_inc2 <- c()
for (i in 1:length(output$mStateIncome)) {
  if (is.na(output$mStateIncome[i])) {
    num_state_med_inc2 <- c(num_state_med_inc2, output$mStateIncome[i])
  } 
  else {
    str_state_med_inc <- as.character(output$mStateIncome[i])
    str_state_med_inc2 <-  unlist(strsplit(str_state_med_inc, ""))
    # Replace the last character with a random digit
    str_state_med_inc2[length(str_state_med_inc2)] <- as.character(state_med_inc_random[i])
    combined_state_med_inc2 <- paste(str_state_med_inc2, collapse = "")
    num_state_med_inc2_cur <- as.numeric(combined_state_med_inc2)
    num_state_med_inc2 <- c(num_state_med_inc2, num_state_med_inc2_cur)
  }

}

# County median income
# Generate a list of random digits used to replace the last digits of the estimates 
county_med_inc_random <- sample(0:9, length(output$mCountyIncome), replace = TRUE)
num_county_med_inc2 <- c()
for (i in 1:length(output$mCountyIncome)) {
  if (is.na(output$mCountyIncome[i])) {
    num_county_med_inc2 <- c(num_county_med_inc2, output$mCountyIncome[i])
  }
  else {
    str_county_med_inc <- as.character(output$mCountyIncome[i])
    str_county_med_inc2 <-  unlist(strsplit(str_county_med_inc, ""))
    # Replace the last character with a random digit
    str_county_med_inc2[length(str_county_med_inc2)] <- as.character(county_med_inc_random[i])
    combined_county_med_inc2 <- paste(str_county_med_inc2, collapse = "")
    num_county_med_inc2_cur <- as.numeric(combined_county_med_inc2)
    num_county_med_inc2 <- c(num_county_med_inc2, num_county_med_inc2_cur)
  }
}

# Tract median income
# Generate a list of random digits used to replace the last digits of the estimates 
tract_med_inc_random <- sample(0:9, length(output$mTractIncome), replace = TRUE)
num_tract_med_inc2 <- c()
for (i in 1:length(output$mTractIncome)) {
  if (is.na(output$mTractIncome[i])) {
    num_tract_med_inc2 <- c(num_tract_med_inc2, output$mTractIncome[i])
  }
  else {
    str_tract_med_inc <- as.character(output$mTractIncome[i])
    str_tract_med_inc2 <-  unlist(strsplit(str_tract_med_inc, ""))
    # Replace the last character with a random digit
    str_tract_med_inc2[length(str_tract_med_inc2)] <- as.character(tract_med_inc_random[i])
    combined_tract_med_inc2 <- paste(str_tract_med_inc2, collapse = "")
    num_tract_med_inc2_cur <- as.numeric(combined_tract_med_inc2)
    num_tract_med_inc2 <- c(num_tract_med_inc2, num_tract_med_inc2_cur)
  }
}

# ZCTA median income
# Generate a list of random digits used to replace the last digits of the estimates 
zcta_med_inc_random <- sample(0:9, length(output$mZCTAIncome), replace = TRUE)
num_zcta_med_inc2 <- c()
for (i in 1:length(output$mZCTAIncome)) {
  if (is.na(output$mZCTAIncome[i])) {
    num_zcta_med_inc2 <- c(num_zcta_med_inc2, output$mZCTAIncome[i])
  }
  else {
    str_zcta_med_inc <- as.character(output$mZCTAIncome[i])
    str_zcta_med_inc2 <-  unlist(strsplit(str_zcta_med_inc, ""))
    # Replace the last character with a random digit
    str_zcta_med_inc2[length(str_zcta_med_inc2)] <- as.character(zcta_med_inc_random[i])
    combined_zcta_med_inc2 <- paste(str_zcta_med_inc2, collapse = "")
    num_zcta_med_inc2_cur <- as.numeric(combined_zcta_med_inc2)
    num_zcta_med_inc2 <- c(num_zcta_med_inc2, num_zcta_med_inc2_cur)
  }
}



# State employment proportion
# Generate a list of random digits used to replace the last digits of the estimates 
state_employment_random <- sample(0:9, length(output$pStateEmployed), replace = TRUE)
num_state_employment2 <- c()
for (i in 1:length(output$pStateEmployed)) {
  if (is.na(output$pStateEmployed[i])) {
    num_state_employment2 <- c(num_state_employment2, output$pStateEmployed[i])
  } 
  else if (output$pStateEmployed[i] == 1) {
    num_state_employment2 <- c(num_state_employment2, 1-state_employment_random[i]/1000000000000000)
  } 
  else {
    str_state_employment <- as.character(output$pStateEmployed[i])
    str_state_employment2 <-  unlist(strsplit(str_state_employment, ""))
    # Replace the last character with a random digit
    str_state_employment2[length(str_state_employment2)] <- as.character(state_employment_random[i])
    combined_state_employment2 <- paste(str_state_employment2, collapse = "")
    num_state_employment2_cur <- as.numeric(combined_state_employment2)
    num_state_employment2 <- c(num_state_employment2, num_state_employment2_cur)
  }
}

# County employment proportion
# Generate a list of random digits used to replace the last digits of the estimates 
county_employment_random <- sample(0:9, length(output$pCountyEmployed), replace = TRUE)
num_county_employment2 <- c()
for (i in 1:length(output$pCountyEmployed)) {
  if (is.na(output$pCountyEmployed[i])) {
    num_county_employment2 <- c(num_county_employment2, output$pCountyEmployed[i])
  } 
  else if (output$pCountyEmployed[i] == 1) {
    num_county_employment2 <- c(num_county_employment2, 1-county_employment_random[i]/1000000000000000)
  } 
  else {
    str_county_employment <- as.character(output$pCountyEmployed[i])
    str_county_employment2 <-  unlist(strsplit(str_county_employment, ""))
    # Replace the last character with a random digit
    str_county_employment2[length(str_county_employment2)] <- as.character(county_employment_random[i])
    combined_county_employment2 <- paste(str_county_employment2, collapse = "")
    num_county_employment2_cur <- as.numeric(combined_county_employment2)
    num_county_employment2 <- c(num_county_employment2, num_county_employment2_cur)
  }
}

# Tract employment proportion
# Generate a list of random digits used to replace the last digits of the estimates 
tract_employment_random <- sample(0:9, length(output$pTractEmployed), replace = TRUE)
num_tract_employment2 <- c()
for (i in 1:length(output$pTractEmployed)) {
  if (is.na(output$pTractEmployed[i])) {
    num_tract_employment2 <- c(num_tract_employment2, output$pTractEmployed[i])
  } 
  else if (output$pTractEmployed[i] == 1) {
    num_tract_employment2 <- c(num_tract_employment2, 1-tract_employment_random[i]/1000000000000000)
  } 
  else {
    str_tract_employment <- as.character(output$pTractEmployed[i])
    str_tract_employment2 <-  unlist(strsplit(str_tract_employment, ""))
    # Replace the last character with a random digit
    str_tract_employment2[length(str_tract_employment2)] <- as.character(tract_employment_random[i])
    combined_tract_employment2 <- paste(str_tract_employment2, collapse = "")
    num_tract_employment2_cur <- as.numeric(combined_tract_employment2)
    num_tract_employment2 <- c(num_tract_employment2, num_tract_employment2_cur)
  }
}

# ZCTA employment proportion
# Generate a list of random digits used to replace the last digits of the estimates 
zcta_employment_random <- sample(0:9, length(output$pZCTAEmployed), replace = TRUE)
num_zcta_employment2 <- c()
for (i in 1:length(output$pZCTAEmployed)) {
  if (is.na(output$pZCTAEmployed[i])) {
    num_zcta_employment2 <- c(num_zcta_employment2, output$pZCTAEmployed[i])
  } 
  else if (output$pZCTAEmployed[i] == 1) {
    num_zcta_employment2 <- c(num_zcta_employment2, 1-zcta_employment_random[i]/1000000000000000)
  } 
  else {
    str_zcta_employment <- as.character(output$pZCTAEmployed[i])
    str_zcta_employment2 <-  unlist(strsplit(str_zcta_employment, ""))
    # Replace the last character with a random digit
    str_zcta_employment2[length(str_zcta_employment2)] <- as.character(zcta_employment_random[i])
    combined_zcta_employment2 <- paste(str_zcta_employment2, collapse = "")
    num_zcta_employment2_cur <- as.numeric(combined_zcta_employment2)
    num_zcta_employment2 <- c(num_zcta_employment2, num_zcta_employment2_cur)
  }
}



# State education proportion
# Generate a list of random digits used to replace the last digits of the estimates 
state_education_random <- sample(0:9, length(output$pStateCollege), replace = TRUE)
num_state_education2 <- c()
for (i in 1:length(output$pStateCollege)) {
  if (is.na(output$pStateCollege[i])) {
    num_state_education2 <- c(num_state_education2, output$pStateCollege[i])
  } 
  else if (output$pStateCollege[i] == 1) {
    num_state_education2 <- c(num_state_education2, 1-state_education_random[i]/1000000000000000)
  } 
  else {
    str_state_education <- as.character(output$pStateCollege[i])
    str_state_education2 <-  unlist(strsplit(str_state_education, ""))
    # Replace the last character with a random digit
    str_state_education2[length(str_state_education2)] <- as.character(state_education_random[i])
    combined_state_education2 <- paste(str_state_education2, collapse = "")
    num_state_education2_cur <- as.numeric(combined_state_education2)
    num_state_education2 <- c(num_state_education2, num_state_education2_cur)
  }
}

# County education proportion
# Generate a list of random digits used to replace the last digits of the estimates 
county_education_random <- sample(0:9, length(output$pCountyCollege), replace = TRUE)
num_county_education2 <- c()
for (i in 1:length(output$pCountyCollege)) {
  if (is.na(output$pCountyCollege[i])) {
    num_county_education2 <- c(num_county_education2, output$pCountyCollege[i])
  } 
  else if (output$pCountyCollege[i] == 1) {
    num_county_education2 <- c(num_county_education2, 1-county_education_random[i]/1000000000000000)
  } 
  else {
    str_county_education <- as.character(output$pCountyCollege[i])
    str_county_education2 <-  unlist(strsplit(str_county_education, ""))
    # Replace the last character with a random digit
    str_county_education2[length(str_county_education2)] <- as.character(county_education_random[i])
    combined_county_education2 <- paste(str_county_education2, collapse = "")
    num_county_education2_cur <- as.numeric(combined_county_education2)
    num_county_education2 <- c(num_county_education2, num_county_education2_cur)
  }
}

# Tract education proportion
# Generate a list of random digits used to replace the last digits of the estimates 
tract_education_random <- sample(0:9, length(output$pTractCollege), replace = TRUE)
num_tract_education2 <- c()
for (i in 1:length(output$pTractCollege)) {
  if (is.na(output$pTractCollege[i])) {
    num_tract_education2 <- c(num_tract_education2, output$pTractCollege[i])
  } 
  else if (output$pTractCollege[i] == 1) {
    num_tract_education2 <- c(num_tract_education2, 1-tract_education_random[i]/1000000000000000)
  } 
  else {
    str_tract_education <- as.character(output$pTractCollege[i])
    str_tract_education2 <-  unlist(strsplit(str_tract_education, ""))
    # Replace the last character with a random digit
    str_tract_education2[length(str_tract_education2)] <- as.character(tract_education_random[i])
    combined_tract_education2 <- paste(str_tract_education2, collapse = "")
    num_tract_education2_cur <- as.numeric(combined_tract_education2)
    num_tract_education2 <- c(num_tract_education2, num_tract_education2_cur)
  }
}

# ZCTA education proportion
# Generate a list of random digits used to replace the last digits of the estimates 
zcta_education_random <- sample(0:9, length(output$pZCTACollege), replace = TRUE)
num_zcta_education2 <- c()
for (i in 1:length(output$pZCTACollege)) {
  if (is.na(output$pZCTACollege[i])) {
    num_zcta_education2 <- c(num_zcta_education2, output$pZCTACollege[i])
  } 
  else if (output$pZCTACollege[i] == 1) {
    num_zcta_education2 <- c(num_zcta_education2, 1-zcta_education_random[i]/1000000000000000)
  } 
  else {
    str_zcta_education <- as.character(output$pZCTACollege[i])
    str_zcta_education2 <-  unlist(strsplit(str_zcta_education, ""))
    # Replace the last character with a random digit
    str_zcta_education2[length(str_zcta_education2)] <- as.character(zcta_education_random[i])
    combined_zcta_education2 <- paste(str_zcta_education2, collapse = "")
    num_zcta_education2_cur <- as.numeric(combined_zcta_education2)
    num_zcta_education2 <- c(num_zcta_education2, num_zcta_education2_cur)
  }
}

perturbed_estimates <- data.frame(cohortPatientId = output$cohortPatientId,
                                  year = output$year,
                                  month = output$month,
                                  day = output$day,
                                  start = output$start,
                                  end = output$end,
                                  record_date = output$record_date,
                                  mStateIncome_perturbed = num_state_med_inc2,
                                  mCountyIncome_perturbed = num_county_med_inc2,
                                  mTractIncome_perturbed = num_tract_med_inc2,
                                  mZCTAIncome_perturbed = num_zcta_med_inc2,
                                  pStateEmployed_perturbed = num_state_employment2,
                                  pCountyEmployed_perturbed = num_county_employment2,
                                  pTractEmployed_perturbed = num_tract_employment2,
                                  pZCTAEmployed_perturbed = num_zcta_employment2,
                                  pStateCollege_perturbed = num_state_education2,
                                  pCountyCollege_perturbed = num_county_education2,
                                  pTractCollege_perturbed = num_tract_education2,
                                  pZCTACollege_perturbed = num_zcta_education2)

write.csv(perturbed_estimates, file = "perturbed_estimates.csv", row.names = FALSE)
