## The GENERATE Tool is created by NA-ACCORD for local address geocoding and 
## submission of area-Level indicators

# Load the necessary libraries
library(dplyr, quietly = TRUE)
library(tidyverse, quietly = TRUE)
library(lubridate, quietly = TRUE)
library(tidyr, quietly = TRUE)
library(stringr)

# Import the last saved output (the temporary output)
output <- read.csv("temp_output.csv")
adrs_geocoded <- read.csv("adrs_geocoded.csv")

# Import the perturbed estiamtes (the last digit is perturbed)
perturbed_estimates <- read.csv("perturbed_estimates.csv")
# Join it together with the main output dataframe
output <- inner_join(output, perturbed_estimates, by = c("cohortPatientId", "year", "month", "day", "start", "end", "record_date"))

#Extract ZCTA
output$zcta <- substr(output$ZCTA_NAME,7,11)

# Extract census tract
output <- separate(output, Tract_NAME, into = c("Census_Tract", "County_dup", "State_dup"), sep=",")
output$censusTract <- ifelse(str_starts(output$Census_Tract, "Census Tract "), str_extract(output$Census_Tract, "(?<=Census Tract ).*"), NA)


## EDITABLE BLOCK ########################################################################################################
# Switches for FIPS at different geography levels of residence
# Local data manager need to check whether they have approval to share the FIPS at each geography level 
# Defaulty is set to sharing all of these
##########################################################################################################################
state_on <- TRUE #If you DON NOT have approval to share state FIPS, please substitute `TRUE` to `FALSE` after `<-`
county_on <- TRUE #If you DON NOT have approval to share county FIPS, please substitute `TRUE` to `FALSE` after `<-`
tract_on <- TRUE #If you DON NOT have approval to share census tract FIPS, please substitute `TRUE` to `FALSE` after `<-`
zcta_on <- TRUE #If you DON NOT have approval to share ZCTA (or ZIP), please substitute `TRUE` to `FALSE` after `<-`
##########################################################################################################################

if (!state_on) {
  output$FIPS_State <- NA
}
if (!county_on) {
  output$FIPS_County <- NA
}
if (!tract_on) {
  output$FIPS_Census_Tract <- NA
}
if (!zcta_on) {
  output$ZCTA <- NA
}

# Rename the geographical variables
output <- output %>% rename(state = State_NAME) 
output <- output %>% rename(county = County_NAME) 
output <- output %>% rename(resDate = record_date) 
output <- output %>% rename(pZCTAEmployee_perturbed = pZCTAEmployed_perturbed) 

# Select and re-arrange the variables in the output tibble
output <- output %>% select(cohortPatientId, resDate, start, end, censusTract, zcta, county, state, 
                            mTractIncome_perturbed, mZCTAIncome_perturbed, mCountyIncome_perturbed, mStateIncome_perturbed,
                            pTractEmployed_perturbed, pZCTAEmployee_perturbed, pCountyEmployed_perturbed, pStateEmployed_perturbed,
                            pTractCollege_perturbed, pZCTACollege_perturbed, pCountyCollege_perturbed, pStateCollege_perturbed
                              )

# Rename the perturbed estimate variables -> remove the "_perturbed" suffix
names(output) <- gsub("_perturbed", "", names(output))

# Arrange the data by patient_id and start date
output <- output %>%
  arrange(cohortPatientId,start)

# Export the output
write.csv(output, file = "output.csv", row.names = FALSE)

